#property script_show_inputs
#property link "https://t.me/fxsaberDiscussion"

#include <MT4Orders.mqh> // https://www.mql5.com/ru/code/16006
#include <..\Files\Graph.mqh> // https://www.mql5.com/en/forum/309903/page2#comment_13264240

#define MAGIC 123454321

input int inHours = 24;
input int inStep = 60; // Step in seconds

struct STAT
{
  datetime time;

  ulong Order;
  ulong Deal;

  ulong ID;

  bool Set( const ulong uDeal )
  {
    this.time = (datetime)::HistoryDealGetInteger(uDeal, DEAL_TIME);

    this.Deal = ::HistoryDealGetInteger(uDeal, DEAL_TICKET);
    this.Order = ::HistoryDealGetInteger(uDeal, DEAL_ORDER);

    this.ID = (ulong)::HistoryDealGetString(uDeal, DEAL_EXTERNAL_ID);

    return((bool)this.Deal);
  }

  void operator -=( const STAT &Stat )
  {
    this.Deal -= Stat.Deal;
    this.Order -= Stat.Order;
    this.ID -= Stat.ID;

    this.time = Stat.time;

    return;
  }

  datetime TimeNorm( const int Step )
  {
    this.time = this.time / Step * Step;

    return(this.time);
  }

  datetime DeleteDate( void )
  {
    return(this.time %= 24 * 3600);
  }

  double Get( const int Type = 0 ) const
  {
    switch (Type)
    {
    case 0:
      return((double)this.Deal);
    case 1:
      return((double)this.Order);
    case 2:
      return((double)this.ID);
    case 3:
      return(this.Deal ? (double)this.Order / this.Deal : 0);
    case 4:
      return(this.Deal ? (double)this.ID / this.Deal : 0);
    case 5:
      return(this.Order ? (double)this.ID / this.Order : 0);
    }

    return(0);
  }

  static const string Str[];
};

static const string STAT::Str[] = {"Deals", "Orders", "IDs", "OrdersDeals", "IDsDeals", "IDsOrders"};

int GetDeals( STAT &Stats[], const int Hours = 24, const MAGIC_TYPE Magic = MAGIC )
{
  datetime LastTime = 0;

  for (int i = OrdersHistoryTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_HISTORY) && (OrderType() <= OP_SELL) && (OrderMagicNumber() == Magic))
    {
      if (!LastTime)
        LastTime = OrderCloseTime();
      else if (LastTime - Hours * 3600 > OrderCloseTime())
        break;

      Stats[ArrayResize(Stats, ArraySize(Stats) + 1) - 1].Set(OrderTicket());
    }

  return(ArraySize(Stats));
}

int GetStatsNorm( STAT &StatsNorm[], const int Hours = 24, const int Step = 60  )
{
  STAT Stats[];
  const int Size = GetDeals(Stats, Hours);

  int Amount = 0;

  if (Size > 1)
  {
    ArrayResize(StatsNorm, Size);

    datetime NextTime = Stats[0].TimeNorm(Step) - Step;
    StatsNorm[0] = Stats[0];

    for (int i = 1; i < Size; i++)
    {
      if (Stats[i].TimeNorm(Step) < NextTime)
      {
       StatsNorm[Amount++] -= Stats[i - 1];
       StatsNorm[Amount] = Stats[i - 1];

       NextTime = Stats[i - 1].time - Step;
      }

      if (i == Size - 1)
       StatsNorm[Amount++] -= Stats[i];
    }
  }

  return(ArrayResize(StatsNorm, Amount));
}

int GetStat( STAT &Stats[], datetime &From, datetime &To, const int Hours = 24, const int Step = 60 )
{
  STAT StatsNorm[];
  datetime Pos[][2];
#define VALUE 0
#define INDEX 1

  const int Size = ArrayResize(Pos, GetStatsNorm(StatsNorm));

  if (Size)
  {
    From = StatsNorm[Size - 1].time;
    To = StatsNorm[0].time;
  }

  for (int i = 0; i < Size; i++)
  {
    Pos[i][VALUE] = StatsNorm[i].DeleteDate();
    Pos[i][INDEX] = i;
  }

  ArraySort(Pos);
  ArrayResize(Stats, Size);

  for (int i = 0; i < Size; i++)
    Stats[i] = StatsNorm[(int)Pos[i][INDEX]];
#undef INDEX
#undef VALUE

  return(Size);
}

bool CreateStatsData( const string FileName, const STAT &Stats[], const datetime From, const datetime To, const int Type = 0 )
{
  const int handle = ::FileOpen(FileName, FILE_WRITE | FILE_TXT | FILE_ANSI);
  const bool Res = (handle != INVALID_HANDLE);

  if (Res)
  {
    const int Size = ArraySize(Stats);

    FileWriteString(handle, "var dat1=[\n");

    uint Amount = 0;

    for (int i = 0; i < Size; i++)
    {
      FileWriteString(handle, "[" + (string)((long)Stats[i].time * 1000) + "," + DoubleToString(Stats[i].Get(Type), 1) + "],\n");

      Amount += (uint)Stats[i].Get(Type);
    }

    FileWriteString(handle, "];\n");
    FileWriteString(handle, "var T1=" + (string)(long)(From * 1000) + ";\n");
    FileWriteString(handle, "var T2=" + (string)(long)(To * 1000) + ";\n");
    FileWriteString(handle, "var nTrades=dat1.length;\n");
    FileWriteString(handle, "var " + STAT::Str[Type] + "=" + (string)Amount + ";\n");
    FileWriteString(handle, "var Currency=\"" + NULL + "\";\n");

    ::FileClose(handle);
  }

  return(Res);
}

void CreateReport(const STAT &Stats[], const datetime From, const datetime To, const int Type = 0 )
{
  static const string Path = __FILE__ + "\\" + AccountInfoString(ACCOUNT_SERVER) + "\\";
  const string FileNameData = STAT::Str[Type] + "Data.txt";

  if (CreateStatsData(Path + FileNameData, Stats, From, To, Type))
  {
    const string FileName = Path + STAT::Str[Type] + ".htm";
    const int handle = ::FileOpen(FileName, FILE_WRITE | FILE_TXT | FILE_ANSI);

    if (handle != INVALID_HANDLE)
    {
      string StrMQH2 = StrMQH;

      StringReplace(StrMQH2, "exdat.txt", FileNameData);
      StringReplace(StrMQH2, "Balance Report", STAT::Str[Type] + ": " + TimeToString(From, TIME_DATE | TIME_MINUTES) + " - "+
                                                                        TimeToString(To, TIME_DATE | TIME_MINUTES));
      StringReplace(StrMQH2, "Balance", STAT::Str[Type]);
      StringReplace(StrMQH2, "trades", "measurements");
      StringReplace(StrMQH2, "Account balance", "Total " + STAT::Str[Type]);

      FileWrite(handle, StrMQH2);

      FileWrite(handle, "<p style=\"text-align: center;\">Details:&nbsp;<a href=\"https://t.me/fxsaberDiscussion\"" +
                        " target=\"_blank\">https://t.me/fxsaberDiscussion</a></p>");
      FileClose(handle);
    }
  }
}

void OnStart()
{
  STAT Stats[];

  datetime From;
  datetime To;

  GetStat(Stats, From, To, inHours, inStep);

  for (int i = 0; i < 6; i++)
    CreateReport(Stats, From, To, i);
}